%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   nls_alphaZ_step3.m -- estimate model using nonlinear least squares
%   this uses the correct method, based on initial values generated by:
%   nls_alphaZ_step1.m and nls_alphaZ_step2.m

%   ALPHA-Z VERSION OF THE MODEL

%   moments to match: for 43 observations, 1963-2005
%   1. skill premium
%   2. skill intensities (2)
%   3. relative price of services

%   feed the model: skill abundance, time, stochastic process for shocks

%   estimate:
%   elasticity of substitution in services
%   elasticity of substitution in goods
%   biased technological change in services (2)
%   biased technological change in goods (2)
%   relative productivity of low-skilled in services (2 parameters)
%   elasticity in demand
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clear global
close all
%clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global data_w                   % skill premium
global data_h                   % skill abundance
global data_hs data_hg          % skill intensitites
global data_p                   % relative price of services
global mu
global T R

load nls_alphaZ_step2

mu=.5 ;                         % distribution parameter on services
T=length(w) ; R=100 ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   SIMULATING SHOCKS
%   last letter id: s=serv, g=goods, a=intersectoral, h=skill abundance

randn('state', 0);  % set seed to avoid unecessary noise
global us ug ua uh

epss = sds*randn(T,R) ; % iid normal shocks: eps ~ N(0,sd^2)
epsg = sdg*randn(T,R) ;
epsa = sda*randn(T,R) ;
epsh = sdh*randn(T,R) ;

%   AR(1) processes: u(t) = r*u(t-1) + eps(t)
us = zeros(T,R) ;
ug = zeros(T,R) ;
ua = zeros(T,R) ;
us(1,:) = epss(1,:) ;   % u(0)=E(u)=0 -> u(1)=eps(1)
ug(1,:) = epsg(1,:) ;
ua(1,:) = epsa(1,:) ;
us(2:T,:) = rs*us(1:T-1,:) + epss(2:T,:) ;
ug(2:T,:) = rg*ug(1:T-1,:) + epsg(2:T,:) ;
ua(2:T,:) = ra*ua(1:T-1,:) + epsa(2:T,:) ;
uh = epsh ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STEP 3: NLS USING MODEL WITH SHOCKS
%   calculate expected values by simulation in every iteration

%   initial values for this search
%   beta_g(t) = exp(beta0g+beta1g*t)
%   beta_s(t) = exp(beta0s+beta1s*t)
%   As_Ag(t) = exp(a0+al*t)
%   [es eg beta0s beta1s beta0g beta1g a0 a1 fi]
%    1  2  3      4      5      6      7  8  9

%%% USE VALUES FROM STEP1:
start=[es eg beta0s beta1s beta0g beta1g a0 a1 fi] ;
start(1:2)=log(start(1:2)) ;    % to keep positive. optimizer takes exponent
start(9)=log(start(9)) ;        % to keep positive. optimizer takes exponent

options = optimset('Display','off') ;

tic
lb=[]; ub=[];
[estimates,out_resnorm,out_residual,out_exitflag,output,lambda,jacobian] = ...
    lsqnonlin(@nls_alphaZ_step3_objective_lsqnonlin,start,lb,ub,options) ;
toc
disp(output)
start(1:2)=exp(start(1:2)) ;
start(9)=exp(start(9)) ;

%   UNPACK ESTIMATES
es=exp(estimates(1)) ;
eg=exp(estimates(2)) ;
beta0s=estimates(3) ;
beta1s=estimates(4) ;
beta0g=estimates(5) ;
beta1g=estimates(6) ;
a0=estimates(7) ;
a1=estimates(8) ;
fi=exp(estimates(9)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   SIMULATE SKILL INTENSITIES AND SKILL PREMIUM USING ESTIMATES

options = optimset('LargeScale','off','TolFun', 0.00001, 'TolX', 0.0001,...
    'MaxIter',100000,'MaxFunEvals',1000000,'Display','off') ;

w=zeros(T,R) ;
hs=zeros(T,R) ;
hg=zeros(T,R) ;
p=zeros(T,R) ;
o=zeros(T,R) ;

global t r
for r=1:1:R  % simulations
    w0=1.5 ; % initial value for fsolve search
    w0=log(w0) ;
    for t=1:1:T % observations
        w(t,r)=fsolve(@nls_alphaZ_step3_equilibrium,w0,options) ;
        w0=w(t,r) ; % update initial value for efficiency in search
        w(t,r)=exp(w(t,r)) ;
        hs(t,r)=(w(t,r)^-es)*exp(beta0s+beta1s*(t-1)+us(t,r))^es ;
        hg(t,r)=(w(t,r)^-eg)*exp(beta0g+beta1g*(t-1)+ug(t,r))^eg ;
        p(t,r)=nls_alphaZ_step3_relprice(w(t,r)) ;   % relative price of S
        o(t,r)=nls_alphaZ_step3_relout(w(t,r)) ;   % relative price of S
    end
    p(:,r)=p(:,r)/p(1,r) ;	% normalize price=1 in 1963, as in data
end

w=mean(w,2) ;
hs=mean(hs,2) ;
hg=mean(hg,2) ;
p=mean(p,2) ;
o=mean(o,2) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   EMPLOYMENT CALCUALTIONS

LshareServ=(data_h-hg)./(hs-hg) ;       % share of L in serv
HshareServ=LshareServ.*hs./data_h ;     % share of H in serv
Hs= HshareServ.*data_h./(1+data_h) ;    % H employed in serv
Hg=(1-HshareServ).*data_h./(1+data_h) ; % H employed in good
Ls=LshareServ./(1+data_h) ;             % L employed in serv
Lg=(1-LshareServ)./(1+data_h) ;         % L employed in good
Es=Hs+Ls ;                              % employment in serv
Eg=Hg+Lg ;                              % employment in good
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   AVERAGE LABOR PRODUCTIVITY

prod_s_g=(p.^-fi).*Eg./Es ;
prod_s_g=prod_s_g/prod_s_g(1) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(1)

estxt=num2str(es,2) ; egtxt=num2str(eg,3) ;

subplot(2,2,1)
plot(data_y,data_hs,'kd--',data_y,hs,'b',data_y,data_hg,'ko--',data_y,hg,'r',...
    data_y,data_h,'k--','LineWidth',2)
%titletxt=['Skill Intensities (EoS_{serv}=' estxt ', EoS_{good}=' egtxt ')'] ;
titletxt=('A. Skill Intensities') ;
title(titletxt)
legend('Services','Simulated','Goods','Simulated','Skill abundance',...
    'Location','NorthWest')
axis([1963 2005 0 max([max(data_hs) max(hs)])+.1])

subplot(2,2,2)
plot(data_y,data_w,'k--',data_y,w,'r','LineWidth',2)
%titletxt=['Skill Premium (EoS_{serv}=' estxt ', EoS_{good}=' egtxt ')'] ;
titletxt=('B. Skill Premium') ;
title(titletxt)
legend('Data','Simulated','Location','NorthWest')
axis([1963 2005 min([min(data_w) min(w)])-.1 max([max(data_w) max(w)])+.1])

subplot(2,2,3)
plot(data_y,data_p,'k--',data_y,p,'r','LineWidth',2)
%titletxt=['Relative Price of Services (EoS_{serv}=' estxt ', EoS_{good}=' egtxt ')'] ;
titletxt=('C. Relative Price of Services') ;
title(titletxt)
legend('Data','Simulated','Location','NorthWest')
axis([1963 2005 min([data_p;p])-.1 max([data_p;p])+.1])

subplot(2,2,4)
plot(data_y,prod_s_g,'LineWidth',2)
title('D. Relative Labor Productivity in Services')
axis([1963 2005 min(prod_s_g)-.1 max(prod_s_g)+.1])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   AGGERGATE ELASTICITY OF SUBSTITUTION

%   ANALYTICAL
EOS_ = smm3_analytical_agg_eos(mu,fi,es,eg,data_h,hs,hg,w) ;
EOS = mean(EOS_) ;

[wgt_es,wgt_eg,wgt_fi] = smm3_analytical_agg_eos_weights(mu,fi,es,eg,data_h,hs,hg,w) ;
EOS_wgt_=es*wgt_es+eg*wgt_eg+fi*wgt_fi ;
EOS_wgt = mean(EOS_wgt_) ;

%   REGRESSIONS
time=1:1:length(data_h) ;
c=ones(length(data_h),1) ;

X=[c log(w) time'] ;
b=inv(X'*X)*X'*log(data_h) ;
agg_est_eos=-b(2) ;
agg_est_trend=b(3) ;
agg_est_btc=b(3)/(-b(2)-1) ;

X=[c log(data_h) time'] ;
b=inv(X'*X)*X'*log(w) ;
agg_est_eos_i=-1/b(2) ;
agg_est_trend_i=b(3) ;
agg_est_btc_i=b(3)/(b(2)+1) ;

X=[c log(data_h) time'] ;
X=X(1:25,:) ;
b=inv(X'*X)*X'*log(w(1:25)) ;
agg_est_eos_i25=-1/b(2) ;
agg_est_trend_i25=b(3) ;
agg_est_btc_i25=b(3)/(b(2)+1) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   DISPLAY RESULTS

out_resnorm_without_price=sum(out_residual(1:3*length(data_w)).^2) ;
disp('es eg beta0s beta1s beta0g beta1g a0 a1 fi')
disp('1  2  3      4      5      6      7  8  9')
disp('    start     ESTIMATES') 
disp([start' [es eg beta0s beta1s beta0g beta1g a0 a1 fi]'])
disp('aggregate analytical eos (average)')
disp(EOS)
disp('out_resnorm')
disp(out_resnorm)
disp('out_resnorm_without_price')
disp(out_resnorm_without_price)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


save nls_alphaZ_step3
