%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   nls_bootstrap.m -- calculate parametric bootstrapped standard errors
%   for model estimates, using nonlinear least squares
%   this uses the correct method, based on initial values generated by:
%   nls_step1.m and nls_step2.m

%   moments to match: for 43 observations, 1963-2005
%   1. skill premium
%   2. skill intensities (2)
%   3. relative price of services

%   feed the model: skill abundance, time, stochastic process for shocks

%   estimate:
%   elasticity of substitution in services
%   elasticity of substitution in goods
%   biased technological change in services (2)
%   biased technological change in goods (2)
%   relative productivity of low-skilled in services (2 parameters)
%   elasticity in demand
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clear global
close all
%clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global T R

load nls_step4

T=length(w) ; R=100 ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   SIMULATING SHOCKS
%   last letter id: s=serv, g=goods, a=intersectoral, h=skill abundance

randn('state', 0);  % set seed to avoid unecessary noise
global us ug ua uh

epss = sds*randn(T,R) ; % iid normal shocks: eps ~ N(0,sd^2)
epsg = sdg*randn(T,R) ;
epsa = sda*randn(T,R) ;
epsh = sdh*randn(T,R) ;

%   AR(1) processes: u(t) = r*u(t-1) + eps(t)
us = zeros(T,R) ;
ug = zeros(T,R) ;
ua = zeros(T,R) ;
us(1,:) = epss(1,:) ;   % u(0)=E(u)=0 -> u(1)=eps(1)
ug(1,:) = epsg(1,:) ;
ua(1,:) = epsa(1,:) ;
us(2:T,:) = rs*us(1:T-1,:) + epss(2:T,:) ;
ug(2:T,:) = rg*ug(1:T-1,:) + epsg(2:T,:) ;
ua(2:T,:) = ra*ua(1:T-1,:) + epsa(2:T,:) ;
uh = epsh ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STEP 3: NLS USING MODEL WITH SHOCKS
%   calculate expected values by simulation in every iteration

%   initial values for this search
%   beta_g(t) = exp(beta0g+beta1g*t)
%   beta_s(t) = exp(beta0s+beta1s*t)
%   As_Ag(t) = exp(a0+al*t)
%   [es eg beta0s beta1s beta0g beta1g a0 a1 fi]
%    1  2  3      4      5      6      7  8  9

%%% USE VALUES FROM STEP1:
start=[es eg beta0s beta1s beta0g beta1g a0 a1 fi] ;
start(1:2)=log(start(1:2)) ;    % to keep positive. optimizer takes exponent
start(9)=log(start(9)) ;        % to keep positive. optimizer takes exponent

options = optimset('Display','off') ;

clear estimates
estimates=zeros(R,length(start)) ;
tic
lb=[]; ub=[];
global r
for r=1:1:R
    disp(r)
    estimates(r,:) = ...
        lsqnonlin(@nls_bootstrap_objective_lsqnonlin,start,lb,ub,options) ;
end
toc
disp(output)
start(1:2)=exp(start(1:2)) ;
start(9)=exp(start(9)) ;

%   STANDARD ERRORS
es_std=std(exp(estimates(:,1))) ;
eg_std=std(exp(estimates(:,2))) ;
beta0s_std=std(estimates(:,3)) ;
beta1s_std=std(estimates(:,4)) ;
beta0g_std=std(estimates(:,5)) ;
beta1g_std=std(estimates(:,6)) ;
a0_std=std(estimates(:,7)) ;
a1_std=std(estimates(:,8)) ;
fi_std=std(exp(estimates(:,9))) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


save nls_bootstrap
