%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   nls_step1.m -- estimate model using nonlinear least squares
%   this uses the inaccurate method for initial values

%   moments to match: for 43 observations, 1963-2005
%   1. skill premium
%   2. skill intensities (2)
%   3. relative price of services

%   feed the model: skill abundance, time

%   estimate:
%   elasticity of substitution in services
%   elasticity of substitution in goods
%   biased technological change in services (2)
%   biased technological change in goods (2)
%   relative productivity of low-skilled in services (2 parameters)
%   elasticity in demand
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clear global
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global data_w                   % skill premium
global data_h                   % skill abundance
global data_hs data_hg          % skill intensitites
global data_p                   % relative price of services
global mu
mu=.5 ;                         % distribution parameter on services
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% data are 43 annual observations, 1963-2005 (CPS 1964-2006)
data=dlmread('data_for_estimation.txt') ;
%data=dlmread('data_for_estimation3.txt') ; % LEE-WOLPIN SECTORS
% unpack variables
data_y=data(:,1) ;              % year
data_w=data(:,2) ;              % skill premium (aggregate)
data_h=data(:,3) ;              % skill adundance
data_hs=data(:,4) ;             % skill intensity in services
data_hg=data(:,5) ;             % skill intensity in goods
clear data
% load relative prices of high-skilled services/goods+low-skilled services
% from BEA: data are for 1963-2005
data_p=dlmread('data_for_estimation_relprice.txt') ; % 1963=1
%data_p=dlmread('data_for_estimation_relprice3.txt') ; % 1963=1 % LEE-WOLPIN SECTORS
data_p=data_p(:,2) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STEP 1: NLS USING MODEL WITH NO SHOCKS
%   this is used only to obtain initial values

%   initial values for this search
%   beta_g(t) = exp(beta0g+beta1g*t)
%   beta_s(t) = exp(beta0s+beta1s*t)
%   As_Ag(t) = exp(a0+al*t)
%   [es eg beta0s beta1s beta0g beta1g a0 a1 fi]
%    1  2  3      4      5      6      7  8  9
start=[0.5 3 2 -0.01 0 0.01 2 0.01 0.01] ;
start(1:2)=log(start(1:2)) ;    % to keep positive. optimizer takes exponent
start(9)=log(start(9)) ;        % to keep positive. optimizer takes exponent

options = optimset('Display','off') ;

tic
lb=[]; ub=[];
[estimates,out_resnorm,out_residual,out_exitflag,output,lambda,jacobian] = ...
    lsqnonlin(@nls_step1_objective_lsqnonlin,start,lb,ub,options) ;
toc
disp(output)
start(1:2)=exp(start(1:2)) ;
start(9)=exp(start(9)) ;

%   UNPACK ESTIMATES
global es eg beta0s beta1s beta0g beta1g a0 a1 fi
es=exp(estimates(1)) ;
eg=exp(estimates(2)) ;
beta0s=estimates(3) ;
beta1s=estimates(4) ;
beta0g=estimates(5) ;
beta1g=estimates(6) ;
a0=estimates(7) ;
a1=estimates(8) ;
fi=exp(estimates(9)) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   SIMULATE SKILL INTENSITIES AND SKILL PREMIUM USING ESTIMATES

options = optimset('LargeScale','off','TolFun', 0.00001, 'TolX', 0.0001,...
    'MaxIter',100000,'MaxFunEvals',1000000,'Display','off') ;

w=zeros(length(data_w),1) ;
hs=zeros(length(data_w),1) ;
hg=zeros(length(data_w),1) ;
p=zeros(length(data_w),1) ;
o=zeros(length(data_w),1) ;

w0=1.5 ; % initial value for fsolve search
w0=log(w0) ;
global t 
for t=1:1:length(data_w)
    w(t,:)=fsolve(@nls_step1_equilibrium,w0,options) ;
    w0=w(t,:) ; % update initial value for efficiency in search
    w(t,:)=exp(w(t,:)) ;
    hs(t,:)=(w(t,:)^-es)*exp(beta0s+beta1s*(t-1))^(es-1) ;
    hg(t,:)=(w(t,:)^-eg)*exp(beta0g+beta1g*(t-1))^(eg-1) ;
    p(t,:)=nls_step1_relprice(w(t,:)) ;   % relative price of S
    o(t,:)=nls_step1_relout(w(t,:)) ;   % relative output S/G
end
p=p/p(1) ;                  % normalize price=1 in 1963, as in data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   EMPLOYMENT CALCUALTIONS

LshareServ=(data_h-hg)./(hs-hg) ;       % share of L in serv
HshareServ=LshareServ.*hs./data_h ;     % share of H in serv
Hs= HshareServ.*data_h./(1+data_h) ;    % H employed in serv
Hg=(1-HshareServ).*data_h./(1+data_h) ; % H employed in good
Ls=LshareServ./(1+data_h) ;             % L employed in serv
Lg=(1-LshareServ)./(1+data_h) ;         % L employed in good
Es=Hs+Ls ;                              % employment in serv
Eg=Hg+Lg ;                              % employment in good
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   AVERAGE LABOR PRODUCTIVITY

prod_s_g=(p.^-fi).*Eg./Es ;
prod_s_g=prod_s_g/prod_s_g(1) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(1)

estxt=num2str(es,2) ; egtxt=num2str(eg,3) ;

subplot(2,2,1)
plot(data_y,data_hs,'kd--',data_y,hs,'b',data_y,data_hg,'ko--',data_y,hg,'r',...
    data_y,data_h,'k--','LineWidth',2)
%titletxt=['Skill Intensities (EoS_{serv}=' estxt ', EoS_{good}=' egtxt ')'] ;
titletxt=('A. Skill Intensities') ;
title(titletxt)
legend('Services','Simulated','Goods','Simulated','Skill abundance',...
    'Location','NorthWest')
axis([1963 2005 0 max([max(data_hs) max(hs)])+.1])

subplot(2,2,2)
plot(data_y,data_w,'k--',data_y,w,'r','LineWidth',2)
%titletxt=['Skill Premium (EoS_{serv}=' estxt ', EoS_{good}=' egtxt ')'] ;
titletxt=('B. Skill Premium') ;
title(titletxt)
legend('Data','Simulated','Location','NorthWest')
axis([1963 2005 min([min(data_w) min(w)])-.1 max([max(data_w) max(w)])+.1])

subplot(2,2,3)
plot(data_y,data_p,'k--',data_y,p,'r','LineWidth',2)
%titletxt=['Relative Price of Services (EoS_{serv}=' estxt ', EoS_{good}=' egtxt ')'] ;
titletxt=('C. Relative Price of Services') ;
title(titletxt)
legend('Data','Simulated','Location','NorthWest')
axis([1963 2005 min([data_p;p])-.1 max([data_p;p])+.1])

subplot(2,2,4)
plot(data_y,prod_s_g,'LineWidth',2)
title('D. Relative Labor Productivity in Services')
axis([1963 2005 min(prod_s_g)-.1 max(prod_s_g)+.1])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   AGGERGATE ELASTICITY OF SUBSTITUTION

%   ANALYTICAL
EOS_ = smm3_analytical_agg_eos(mu,fi,es,eg,data_h,hs,hg,w) ;
EOS = mean(EOS_) ;

[wgt_es,wgt_eg,wgt_fi] = smm3_analytical_agg_eos_weights(mu,fi,es,eg,data_h,hs,hg,w) ;
EOS_wgt_=es*wgt_es+eg*wgt_eg+fi*wgt_fi ;
EOS_wgt = mean(EOS_wgt_) ;

%   REGRESSIONS
time=1:1:length(data_h) ;
c=ones(length(data_h),1) ;

X=[c log(w) time'] ;
b=inv(X'*X)*X'*log(data_h) ;
agg_est_eos=-b(2) ;
agg_est_trend=b(3) ;
agg_est_btc=b(3)/(-b(2)-1) ;

X=[c log(data_h) time'] ;
b=inv(X'*X)*X'*log(w) ;
agg_est_eos_i=-1/b(2) ;
agg_est_trend_i=b(3) ;
agg_est_btc_i=b(3)/(b(2)+1) ;

X=[c log(data_h) time'] ;
X=X(1:25,:) ;
b=inv(X'*X)*X'*log(w(1:25)) ;
agg_est_eos_i25=-1/b(2) ;
agg_est_trend_i25=b(3) ;
agg_est_btc_i25=b(3)/(b(2)+1) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   DISPLAY RESULTS

out_resnorm_without_price=sum(out_residual(1:3*length(data_w)).^2) ;
disp('es eg beta0s beta1s beta0g beta1g a0 a1 fi')
disp('1  2  3      4      5      6      7  8  9')
disp('    start     ESTIMATES') 
disp([start' [es eg beta0s beta1s beta0g beta1g a0 a1 fi]'])
disp('aggregate analytical eos (average)')
disp(EOS)
disp('out_resnorm')
disp(out_resnorm)
disp('out_resnorm_without_price')
disp(out_resnorm_without_price)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   CORRELATION COEFFICIENTS AND STANDARD DEVIATIONS FOR AR(1)
%   to be used in step 2
%   this is not the residuals in the output series

u_w=(log(data_w)-log(w)) ;
u_hs=(log(data_hs)-log(hs)) ;
u_hg=(log(data_hg)-log(hg)) ;
u_p=(log(data_p)-log(p)) ;

corr_u_w=corr(u_w(1:42),u_w(2:43)) ;  % correlation coefficients
corr_u_hs=corr(u_hs(1:42),u_hs(2:43)) ;
corr_u_hg=corr(u_hg(1:42),u_hg(2:43)) ;
corr_u_p=corr(u_p(1:42),u_p(2:43)) ;

std_u_w=std(u_w) ;
std_u_hs=std(u_hs) ;
std_u_hg=std(u_hg) ;
std_u_p=std(u_p) ;

std_e_w = std_u_w*(1-corr_u_w^2)^.5 ;
std_e_hs = std_u_hs*(1-corr_u_hs^2)^.5 ;
std_e_hg = std_u_hg*(1-corr_u_hg^2)^.5 ;
std_e_p = std_u_p*(1-corr_u_p^2)^.5 ;

figure(2)

subplot(2,2,1)
plot(data_y,u_w); title('u w'); axis([1963 2005 min(u_w) max(u_w)])
subplot(2,2,2)
plot(data_y,u_hs); title('u hs'); axis([1963 2005 min(u_hs) max(u_hs)])
subplot(2,2,3)
plot(data_y,u_hg); title('u hg'); axis([1963 2005 min(u_hg) max(u_hg)])
subplot(2,2,4)
plot(data_y,u_p); title('u p'); axis([1963 2005 min(u_p) max(u_p)])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


save nls_step1
